'use client';

import { useState } from 'react';
import { Check, ChevronDownIcon } from 'lucide-react';

import { cn } from '@/lib/utils';

import { Button } from '../ui/button';
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem, CommandList } from '../ui/command';
import { Popover, PopoverContent, PopoverTrigger } from '../ui/popover';

interface ComboBoxProps<Option> {
  items?: Option[];
  value: Option | null;
  onChange: (newValue: Option | null) => void;

  filterFunc: (item: Option, query: string) => boolean;

  idFunc: (item: Option) => string;
  labelValueFunc: (item: Option) => string;
  labelOptionFunc: (item: Option) => string;

  placeholder?: string;
  className?: string;
  noBorder?: boolean;
}

/**
 * Displays a combo-select component.
 */
export function ComboBox<Option>({
  items,
  filterFunc,
  value,
  onChange,
  labelValueFunc,
  labelOptionFunc,
  idFunc,
  noBorder,
  placeholder,
  className
}: ComboBoxProps<Option>) {
  const [open, setOpen] = useState(false);
  const [query, setQuery] = useState('');

  const filtered = items?.filter(item => filterFunc(item, query)) ?? [];

  return (
    <Popover open={open} onOpenChange={setOpen}>
      <PopoverTrigger asChild>
        <Button
          variant='ghost'
          role='combobox'
          aria-expanded={open}
          className={cn(
            'justify-between font-normal hover:bg-input',
            "[&_svg:not([class*='text-'])]:text-muted-foreground",
            open && 'cursor-auto',
            !noBorder && 'border',
            !value && 'text-muted-foreground hover:text-muted-foreground',
            className
          )}
        >
          <span className='truncate'>{value ? labelValueFunc(value) : placeholder}</span>
          <ChevronDownIcon />
        </Button>
      </PopoverTrigger>
      <PopoverContent className='p-0'>
        <Command>
          <CommandInput value={query} onValueChange={setQuery} placeholder='Поиск...' className='h-9' />
          <CommandList>
            <CommandEmpty>Список пуст</CommandEmpty>
            <CommandGroup>
              {filtered.map(item => (
                <CommandItem
                  key={idFunc(item)}
                  value={idFunc(item)}
                  onSelect={() => {
                    onChange(item);
                    setOpen(false);
                  }}
                >
                  {labelOptionFunc(item)}
                  <Check className={cn('ml-auto', value === item ? 'opacity-100' : 'opacity-0')} />
                </CommandItem>
              ))}
            </CommandGroup>
          </CommandList>
        </Command>
      </PopoverContent>
    </Popover>
  );
}
